@@ -1,5 +1,5 @@
 /*  Lzlib - Compression library for the lzip format
-    Copyright (C) 2009-2018 Antonio Diaz Diaz.
+    Copyright (C) 2009-2019 Antonio Diaz Diaz.
 
     This library is free software. Redistribution and use in source and
     binary forms, with or without modification, are permitted provided
@@ -58,9 +58,9 @@ static inline bool FLZe_init( struct FLZ_encoder * const fe,
          /* bytes to keep in buffer after pos */
          after_size = max_match_len,
          dict_factor = 16,
+         min_free_bytes = max_marker_size,
          num_prev_positions23 = 0,
-         pos_array_factor = 1,
-         min_free_bytes = max_marker_size };
+         pos_array_factor = 1 };
 
   return LZeb_init( &fe->eb, before_size, dict_size, after_size, dict_factor,
                     num_prev_positions23, pos_array_factor, min_free_bytes,
